#include <GL/gl.h>


enum {
   ZERO = 4, 
   ONE = 5
};



#define SWZ_CPY(dst, src, count, dstComps, srcComps) \
   do { \
      GLuint i; \
      for (i = 0; i < count; i++) { \
         GLuint j; \
         for (j = 0; j < srcComps; j++) { \
            tmp[j] = src[j]; \
         } \
	 src += srcComps; \
         for (j = 0; j < dstComps; j++) { \
            dst[j] = tmp[map[j]]; \
         } \
	 dst += dstComps; \
      } \
   } while (0)



/**
 * Copy GLubyte pixels from <src> to <dst> with swizzling.
 * \param dst  destination pixels
 * \param dstComponents  number of color components in destination pixels
 * \param src  source pixels
 * \param srcComponents  number of color components in source pixels
 * \param map  the swizzle mapping.  map[X] says where to find the X component
 *             in the source image's pixels.  For example, if the source image
 *             is GL_BGRA and X = red, map[0] yields 2.
 * \param count  number of pixels to copy/swizzle.
 */
void
swizzle_copy(GLubyte *dst, GLuint dstComponents, const GLubyte *src, 
             GLuint srcComponents, const GLubyte *map, GLuint count)
{
   GLubyte tmp[6];
   GLuint i;

   tmp[ZERO] = 0x0;
   tmp[ONE] = 0xff;

   switch (dstComponents) {
   case 4:
      switch (srcComponents) {
      case 4:
         /* special-case / use COPY_4UBV */
         for (i = 0; i < count; i++) {
            COPY_4UBV(tmp, src); 
            src += srcComponents;      
            dst[0] = tmp[map[0]];
            dst[1] = tmp[map[1]];
            dst[2] = tmp[map[2]];
            dst[3] = tmp[map[3]];
            dst += 4;
         }
         break;
      case 3:
         SWZ_CPY(dst, src, count, 4, 3);
         break;
      case 2:
         SWZ_CPY(dst, src, count, 4, 2);
         break;
      case 1:
         SWZ_CPY(dst, src, count, 4, 1);
         break;
      default:
         ;
      }
      break;
   case 3:
      switch (srcComponents) {
      case 4:
         SWZ_CPY(dst, src, count, 3, 4);
         break;
      case 3:
         SWZ_CPY(dst, src, count, 3, 3);
         break;
      case 2:
         SWZ_CPY(dst, src, count, 3, 2);
         break;
      case 1:
         SWZ_CPY(dst, src, count, 3, 1);
         break;
      default:
         ;
      }
      break;
   case 2:
      switch (srcComponents) {
      case 4:
         SWZ_CPY(dst, src, count, 2, 4);
         break;
      case 3:
         SWZ_CPY(dst, src, count, 2, 3);
         break;
      case 2:
         SWZ_CPY(dst, src, count, 2, 2);
         break;
      case 1:
         SWZ_CPY(dst, src, count, 2, 1);
         break;
      default:
         ;
      }
      break;
   case 1:
#if 0
      /* XXX investigate valgrind invalid read when running demos/texenv.c */
      for (i = 0; i < count; i++) {
 	 COPY_4UBV(tmp, src); 
	 src += srcComponents;      
	 dst[0] = tmp[map[0]];
	 dst += 1;
      }
#else
      switch (srcComponents) {
      case 4:
         SWZ_CPY(dst, src, count, 1, 4);
         break;
      case 3:
         SWZ_CPY(dst, src, count, 1, 3);
         break;
      case 2:
         SWZ_CPY(dst, src, count, 1, 2);
         break;
      case 1:
         SWZ_CPY(dst, src, count, 1, 1);
         break;
      default:
         ;
      }
      break;
#endif
      break;
   }
}

